// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
//	boresight:	test retrieval of boresight angles
//
#include <iostream>

#include "CalServer.h"
#include "Boresight.h"
#include "Celestial.h"

using namespace std;

int main()
{
	calServer.setInstrument(Xmm::EMOS1);
	calServer.state()->ccd()->set(EMOScam::CCD_1);
	calServer.state()->node()->set(EMOScam::PRIMARY_NODE);

	Boresight *bs = 0; bs = calServer.getAtom(bs);
	AttitudeMatrix A(bs->matrix(0.));

	CartesianVector v(1., 0., 0.),
					vres = A*v;
	cout<<vres.x()<<" "<<vres.y()<<" "<<vres.z()<<endl;

        cout<<"Displaying MOS angles"<<endl;
        cout<<"Angles after time set:\n"<<bs->angles(168164754.627751)
                <<"\nand as matrix:\n"<<bs->matrix(168164754.627751)<<endl;
	calServer.setInstrument(Xmm::OM);
        cout<<"Displaying OM angles"<<endl;
	Boresight *bs2 = 0; bs2 = calServer.getAtom(bs2);
	cout<<"Angles after setting:\n"<<bs2->angles(0.)
		<<"\nand as matrix:\n"<<bs2->matrix(0.)<<endl;

       // Change the time of the boresight
        cout<<"Angles after time set:\n"<<bs2->angles(168164754.627751)
                <<"\nand as matrix:\n"<<bs2->matrix(168164754.627751)<<endl;

	calServer.setInstrument(Xmm::EMOS1);
        cout<<"Displaying EMOS1 angles"<<endl;
	bs2 = calServer.getAtom(bs2);
	cout<<"Angles after setting:\n"<<bs2->angles(0.)
		<<"\nand as matrix:\n"<<bs2->matrix(0.)<<endl;

       // Change the time of the boresight
        cout<<"Angles after time set:\n"<<bs2->angles(63712280.1840171)
                <<"\nand as matrix:\n"<<bs2->matrix(63712280.1840171)<<endl;

	calServer.setInstrument(Xmm::EMOS2);
        cout<<"Displaying EMOS2 angles"<<endl;
	bs2 = calServer.getAtom(bs2);
	cout<<"Angles after setting:\n"<<bs2->angles(0.)
		<<"\nand as matrix:\n"<<bs2->matrix(0.)<<endl;

       // Change the time of the boresight
        cout<<"Angles after time set:\n"<<bs2->angles(169164754.627751)
                <<"\nand as matrix:\n"<<bs2->matrix(169164754.627751)<<endl;

	calServer.setInstrument(Xmm::EPN);
        cout<<"Displaying EPN angles"<<endl;
	bs2 = calServer.getAtom(bs2);
	cout<<"Angles after setting:\n"<<bs2->angles(0.)
		<<"\nand as matrix:\n"<<bs2->matrix(0.)<<endl;

       // Change the time of the boresight
        cout<<"Angles after time set:\n"<<bs2->angles(188164754.627751)
                <<"\nand as matrix:\n"<<bs2->matrix(188164754.627751)<<endl;

	return 0;
}
