// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
//	test the rgs thresholds
//
#include <iostream>
#include <cmath>
#include "CalServer.h"
#include "EnergyCorrector.h"
#include "Xmm.h"

using namespace std;

#if 0
static bool isClose(double a, double b)
{
    return fabs(a)/fabs(b)-1. < 1.e-5;
}
#endif


static void process(XmmInstrument &rgsunit)
{
	cerr<<"Instrument: "<<Xmm::toString(rgsunit.id())<<endl;
	calServer.setInstrument(rgsunit.id());

	for (unsigned ccd = 1; ccd<=rgsunit.nCcds(); ++ccd) {
		calServer.state()->ccd()->set(ccd);
		EnergyCorrector *ec = 0; ec = calServer.getAtom(ec);

		const vector<Node> &nodes = rgsunit.nodes();
		for (unsigned nodeId = 0; nodeId < nodes.size(); ++nodeId) {
			cerr<<"thresholds for CCD #"<<ccd<<" node "
				<<nodes[nodeId]<<endl;
			cerr<<"rejection: ";
			for (unsigned grade=1; grade<ec->nGrades(); ++grade)
				cerr<<ec->rejectionThreshold(grade, nodes[nodeId])
					<<" ";
			cerr<<"\nacceptance: ";
			for (unsigned grade=1; grade<ec->nGrades(); ++grade)
				cerr<<ec->acceptanceThreshold(grade, nodes[nodeId])
					<<" ";
			cerr<<endl;
		}
	}

	calServer.state()->ccd()->set(1);
	calServer.state()->exposureStartTime()->
				set(STime("2000-10-05T16:45:12"));

	double t;
	int16 x, y, sep;
	int8 sh;
	int16 einint;
	real32 e;
	CalReal64Vector time(&t, 1);
	CalInt8Vector shv(&sh, 1);
	CalInt16Vector xv(&x, 1), yv(&y, 1), sv(&sep, 1), einintv(&einint, 1);
	CalReal32Vector ein(&e, 1), eout(&e, 1);
	t = 87151576.184;	//	2000-10-05T16:45:12
	sh = 0;
	x = y = sep = 1;
	e = 0.;
		
	EnergyCorrector *offCorr = 0; offCorr = calServer.getAtom(offCorr);
	cout<<"Offset correction: "<<endl;
	einint = 0;
	offCorr->offsetCorrect(time, xv, yv, shv, einintv, eout);
	cout<<"E in : "<<einintv[0]<<endl;
	cout<<"E out: "<<eout[0]<<endl;

	CtiCorrector *cti = 0; cti = calServer.getAtom(cti);
	cti->ctiCorrect(time, xv, yv, shv, sv, ein, eout, 0);
}


int main()
{
	process(xmmPayLoad.rgs1);
	process(xmmPayLoad.rgs2);

	return 0;
}

