// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
//	test RGS pixel size in OCB 3x3 / ROWCOORDs; print ROWCOORD coordinates
//	of outer corners of first N (cmd line arg) pixel rows/columns
//
#include <iostream>
#include <iomanip>
#include <cmath>
#include <cstdlib>
#include "CalServer.h"
#include "Xmm.h"
#include "GeometryDataServer.h"
#include "common.h"

using namespace std;

namespace {
	bool isClose(double a, double b) {
		double d = fabs(fabs(a/b)-1.);
//		cout<<setprecision(12)<<a<<'/'<<b<<' '<<d<<endl;
		return d < 1.e-2;
	}
}

int main(int, char **)
{
	calServer.setInstrument(Xmm::RGS1);
	calServer.state()->randomize()->set(false);
	calServer.state()->ccd()->set(5);
	calServer.state()->ocb()->set(3);

	CalInt16Vector	rawXVec(0, 1),
				    rawYVec(0, 1);
	rawXVec[0] = 1;
	rawYVec[0] = 1;

	GeometryDataServer *cd = 0; cd = calServer.getAtom(cd);
	XmmPixelList pl = cd->chipCoordList(rawXVec, rawYVec);
	XmmCartesianPointList cpl = pl.toCamCoord1();

	cout<<setprecision(12)<<cpl.x()[0]<<' '<<cpl.y()[0]<<' '
						  <<cpl.z()[0]<<endl;

	return 0;
}
