! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
!   epntime.f90  --- test fine time adjustments for EPN
!
program epntime

use cal
use types
use caloaldefs
use errorhandling

    integer                             :: mode, ccd, i
    real(double), dimension(0:0)        :: time
    integer(kind=int16), dimension(0:0) :: rawY
    integer(kind=int16)                 :: sourceY
    logical                             :: isClose
    real(double), dimension(0:40)       :: nomTime = (/ &
      -51.2338600158691406, -45.1234588623046946, -39.0132598876953125, &
      -51.2338600158691406, -45.1234588623046946, -39.0132598876953125, &
      -51.2338600158691406, -45.1234588623046946, -39.0132598876953125, &
      -51.2338600158691406, -45.1234588623046946, -39.0132598876953125, &
      -1.1414842224121094E+2, -1.0803801727294922E+2, -1.0192781829833984E+2, &
      -1.1414842224121094E+2, -1.0803801727294922E+2, -1.0192781829833984E+2, &
      -1.1414842224121094E+2, -1.0803801727294922E+2, -1.0192781829833984E+2, &
      -1.1414842224121094E+2, -1.0803801727294922E+2, -1.0192781829833984E+2, &
      -33.0299186706542969, -29.0611591339111328, -25.0925998687744141, &
      -33.0299186706542969, -29.0611591339111328, -25.0925998687744141, &
      -33.0299186706542969, -29.0611591339111328, -25.0925998687744141, &
      -33.0299186706542969, -29.0611591339111328, -25.0925998687744141, &
      -3.6809599399566650, -4.2797598838806152, -1.4779999852180481, &
      -2.3943599760532379, -4.1782400925876573/)

    call CAL_setState(EPN)

    !   test three main imaging modes
    i = 0

    mode = PRIME_FULL_WINDOW
    write(*, *) 'In mode: ', modeId(mode)
    call CAL_setState(ccdModeId=mode)
    do ccd=EPN_CCD_FIRST, EPN_CCD_LAST
        call CAL_setState(ccdChipId=ccd)
        time(0) = 0
        call CAL_correctTime(time)
        write(*, *) 'CCD ', ccd, ' time=', 1000.*time(0), ' ms'
        if (.not. isClose(1000.*time(0), nomTime(i))) &
             call error("Something wrong with fine time")
        i = i + 1
    enddo

    do mode=PRIME_FULL_WINDOW_EXTENDED, PRIME_LARGE_WINDOW
        write(*, *) 'In mode: ', modeId(mode)
        call CAL_setState(ccdModeId=mode)
        do ccd=EPN_CCD_FIRST, EPN_CCD_LAST
            call CAL_setState(ccdChipId=ccd)
            time(0) = 0
            call CAL_correctTime(time)
            write(*, *) 'CCD ', ccd, ' time=', 1000.*time(0), ' ms'
!            if (.not. isClose(1000.*time(0), nomTime(i))) &
!                 call error("Something wrong with fine time")
            i = i + 1
        enddo
    enddo



    !   test small window mode - only CCD 4 can be active
    mode = PRIME_SMALL_WINDOW
    call CAL_setState(ccdModeId=mode, ccdChipId=EPN_CCD_4)
    time(0) = 0.
    call CAL_correctTime(time)
    write(*, *) 'In mode: ', modeId(mode)
    write(*, *) 'CCD ', EPN_CCD_4, ' time=', 1000.*time(0), ' ms'
    if (.not. isClose(1000.*time(0), nomTime(i))) &
             call error("Something wrong with fine time")
    i = i + 1

    !   burst mode
    mode = FAST_BURST
    call CAL_setState(ccdModeId=mode)
    time(0) = 0.
    call CAL_correctTime(time)
    write(*, *) 'In mode: ', modeId(mode)
    write(*, *) 'CCD ', EPN_CCD_4, ' time=', 1000.*time(0), ' ms'
    if (.not. isClose(1000.*time(0), nomTime(i))) &
             call error("Something wrong with fine time")
    i = i + 1

    !   timing and burst mode with and without PSF position
    rawY(0) = 129
    write(*, *) 'Timing mode without source position, rawY =', rawY(0)
    mode = FAST_TIMING
    call CAL_setState(ccdModeId = mode)
    write(*, *) 'In mode: ', modeId(mode)
    time(0) = 0.
    call CAL_correctTime(time, rawY)
    write(*, *) 'time=', 1000.*time(0), ' ms'
    if (.not. isClose(1000.*time(0), nomTime(i))) &
             call error("Something wrong with fine time")
    i = i + 1

    sourceY = 100
    write(*, *) 'Timing/burst mode: rawY=', rawY(0), 'src pos=', sourceY
    do mode=FAST_TIMING, FAST_BURST
        call CAL_setState(ccdModeId = mode)
        write(*, *) 'In mode: ', modeId(mode)
        time(0) = 0.
        call CAL_correctTime(time, rawY, sourceY)
        write(*, *) 'time=', 1000.*time(0), ' ms'
        if (.not. isClose(1000.*time(0), nomTime(i))) &
                 call error("Something wrong with fine time")
        i = i + 1
    enddo

    call message(AppMsg, Verbose,"all tests passed")
end program


logical function isClose(value, trueval)
use types

    real(double), intent(in)    :: value, trueval
    real(double)                :: absval, abstrueval

    isClose = .true.
    if (value * trueval .lt. 0.) isClose = .false.
    absval = abs(value)
    abstrueval = abs(trueval)
    if (absval .lt. .99999 * abstrueval .or. &
        absval .gt. 1.0001 * abstrueval) isClose = .false.
end function

